Package: enrichplot
Title: Visualization of Functional Enrichment Result
Version: 1.30.2
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person(given = "Chun-Hui", family = "Gao", email = "gaospecial@gmail.com", role = "ctb", comment = c(ORCID = "0000-0002-1445-7939")))
Description: The 'enrichplot' package implements several visualization methods for interpreting functional enrichment results obtained from ORA or GSEA analysis. 
    It is mainly designed to work with the 'clusterProfiler' package suite. All the visualization methods are developed based on 'ggplot2' graphics.
Depends: R (>= 4.1.0)
Imports: aplot (>= 0.2.1), DOSE (>= 3.31.2), ggfun (>= 0.1.7),
        ggnewscale, ggplot2 (>= 3.5.0), ggrepel (>= 0.9.0), ggtangle
        (>= 0.0.5), graphics, grid, igraph, methods, plyr, purrr,
        RColorBrewer, reshape2, rlang, stats, utils, scatterpie,
        GOSemSim (>= 2.31.2), magrittr, ggtree, yulab.utils (>= 0.1.6)
Suggests: clusterProfiler, dplyr, europepmc, ggarchery, ggupset, glue,
        knitr, rmarkdown, org.Hs.eg.db, prettydoc, tibble, tidyr,
        ggforce, ggHoriPlot, AnnotationDbi, ggplotify, ggridges,
        grDevices, gridExtra, ggstar, scales, ggtreeExtra, tidydr
VignetteBuilder: knitr
License: Artistic-2.0
URL: https://yulab-smu.top/contribution-knowledge-mining/
BugReports: https://github.com/GuangchuangYu/enrichplot/issues
biocViews: Annotation, GeneSetEnrichment, GO, KEGG, Pathways, Software,
        Visualization
Encoding: UTF-8
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/enrichplot
git_branch: RELEASE_3_22
git_last_commit: e85d5d3
git_last_commit_date: 2025-11-10
Repository: Bioconductor 3.22
Date/Publication: 2025-11-11
NeedsCompilation: no
Packaged: 2025-11-11 22:35:18 UTC; biocbuild
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Chun-Hui Gao [ctb] (ORCID: <https://orcid.org/0000-0002-1445-7939>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
