#' @importFrom ggplot2 ggtitle
#' @export
ggplot2::ggtitle

#' @importFrom ggplot2 facet_grid
#' @export
ggplot2::facet_grid

#' @importFrom aplot plot_list
#' @export
aplot::plot_list

#' @importFrom ggtangle cnetplot
#' @export
ggtangle::cnetplot


#' @importFrom ggtangle geom_cnet_label
#' @export 
ggtangle::geom_cnet_label

