---
title: "Visualization of Functional Enrichment Result"
author: "\\

	Guangchuang Yu\\

        School of Basic Medical Sciences, Southern Medical University"
date: "`r Sys.Date()`"
output:
  prettydoc::html_pretty:
    toc: true
    theme: cayman
    highlight: github
  pdf_document:
    toc: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{enrichplot}
  %\usepackage[utf8]{inputenc}
---

Please go to <https://yulab-smu.github.io/clusterProfiler-book/> for the full vignette.