% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{color_palette}
\alias{color_palette}
\title{color_palette}
\usage{
color_palette(colors)
}
\arguments{
\item{colors}{colors of length >=2}
}
\value{
color vector
}
\description{
create color palette for continuous data
}
\examples{
color_palette(c("red", "yellow", "green"))
}
\author{
guangchuang yu
}
